package org.qatools.validator;

import java.lang.annotation.Annotation;
import java.lang.reflect.Method;


/**
 *
 */
public class StackUnwinder {

    private static final String TEST_METHOD_NAME_PREFIX = "test";
    private static final String JUNIT_ANNOTATION_NAME = "org.junit.Test";

    private StackUnwinder() {
    }


    /**
     * Returns information about test method, which called current method
     *
     * @return nearest JUnit test method details or NULL if not found
     */
    public static StackTraceElement getCallerTestMethodDetails() {
        StackTraceElement[] stackTrace = Thread.currentThread().getStackTrace();
        for (StackTraceElement stackTraceElement : stackTrace) {
            if (isTestMethodByAnnotation(stackTraceElement)) {
                //log.debug("Detected method with @Test annotation");
                return stackTraceElement;
            }
            if (isTestMethodByName(stackTraceElement)) {
                //log.debug("Detected method with 'test' prefix");
                return stackTraceElement;
            }
        }
        return null;
    }

    public static StackTraceElement getActualCallerDetails() {
        StackTraceElement[] stackTrace = Thread.currentThread().getStackTrace();
        return stackTrace[3];
    }

    /**
     * Checks if stack trace element is test case using expected JUnit annotation @Test
     *
     * @param stackTraceElement stack trace element to analyze
     * @return true if stack trace element is test method
     */
    private static boolean isTestMethodByAnnotation(StackTraceElement stackTraceElement) {
        String className = stackTraceElement.getClassName();
        String methodName = stackTraceElement.getMethodName();
        try {
            // getting class
            Class<?> clazz = Class.forName(className);
            //getting methods
            Method[] methods = clazz.getMethods();
            for (Method method : methods) {
                method.setAccessible(true);
                // there can't be 2 methods with the same name in test class as test methods don't have parameters
                if (method.getName().equals(methodName)) {
                    // getting annotations
                    Annotation[] annotations = method.getAnnotations();
                    for (Annotation annotation : annotations) {
                        // checking annotation
                        String annotationName = annotation.annotationType().getName();
                        if (JUNIT_ANNOTATION_NAME.equals(annotationName)) {
                            return true;
                        }
                    }
                }
            }
            return false;
        } catch (Exception e) {
            String msg = "Failed to detect method annotations. Error: " + e;
            //log.error(msg, e);
            //todo: maybe exception should be re-thrown here instead
            return false;
        }
    }

    /**
     * Checks if stack trace element is test case using expected name prefix
     *
     * @param stackTraceElement stack trace element to analyze
     * @return true if stack trace element is test method
     */
    private static boolean isTestMethodByName(StackTraceElement stackTraceElement) {
        String methodName = stackTraceElement.getMethodName();
        return methodName.startsWith(TEST_METHOD_NAME_PREFIX);
    }

}

